package cn.wzpmc.commerce.services;

import cn.wzpmc.commerce.entities.Group;
import cn.wzpmc.commerce.entities.Permission;
import cn.wzpmc.commerce.entities.User;
import cn.wzpmc.commerce.entities.vo.AccountVo;
import cn.wzpmc.commerce.entities.vo.GroupVo;
import cn.wzpmc.commerce.entities.vo.PermissionVo;
import cn.wzpmc.commerce.mappers.AccountMapper;
import cn.wzpmc.commerce.mappers.GroupMapper;
import cn.wzpmc.commerce.mappers.PermissionsMapper;
import cn.wzpmc.commerce.utils.EntitiesUtils;
import cn.wzpmc.commerce.utils.MessageResponse;
import cn.wzpmc.commerce.utils.PageableResponse;
import cn.wzpmc.commerce.utils.ServiceUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Service
public class UserService extends ServiceImpl<AccountMapper, AccountVo> implements UserDetailsService {
    private final AccountMapper accountMapper;
    private final PermissionsMapper permissionsMapper;
    private final GroupMapper groupMapper;
    private final PasswordEncoder encoder;
    private final Map<Integer, SecurityContext> userContexts = new HashMap<>();

    @Autowired
    public UserService(AccountMapper accountMapper, PermissionsMapper permissionsMapper, GroupMapper rolesMapper, PasswordEncoder encoder) {
        this.accountMapper = accountMapper;
        this.permissionsMapper = permissionsMapper;
        this.groupMapper = rolesMapper;
        this.encoder = encoder;
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        SecurityContext context = SecurityContextHolder.getContext();
        User userByName = getUserByName(username);
        userContexts.put(userByName.getId(), context);
        return userByName;
    }

    public User addUser(AccountVo accountVo) {
        try {
            this.loadUserByUsername(accountVo.getUsername());
        } catch (UsernameNotFoundException e) {
            accountVo.setPassword(this.encoder.encode(accountVo.getPassword()));
            accountVo.setCreateTime(new Date());
            this.accountMapper.insert(accountVo);
            return this.getUserByName(accountVo.getUsername());
        }
        return null;
    }

    private User getUserByName(String username) throws UsernameNotFoundException {
        AccountVo accountVo = this.accountMapper.selectOne(new QueryWrapper<AccountVo>()
                .eq("username", username)
        );
        if (accountVo == null) {
            throw new UsernameNotFoundException("用户不存在");
        }
        return genUserFromAuthAccount(accountVo);
    }

    private User getUserById(int id) throws UsernameNotFoundException {
        AccountVo accountVo = this.accountMapper.selectById(id);
        if (accountVo == null) {
            throw new UsernameNotFoundException("用户不存在");
        }
        return genUserFromAuthAccount(accountVo);
    }

    private User genUserFromAuthAccount(AccountVo accountVo) {
        User user = EntitiesUtils.copyField(accountVo, new User());
        GroupVo groupVo = groupMapper.selectById(accountVo.getGroupId());
        Group group = EntitiesUtils.copyField(groupVo, new Group());
        group.setPermissions(permissionsMapper.selectList(
                new QueryWrapper<PermissionVo>().eq("group_id", groupVo.getId())
        ).stream().map(e -> EntitiesUtils.copyField(e, new Permission())).toList());
        user.setGroup(group);
        return user;
    }

    public MessageResponse changeAccount(AccountVo authAccountVo) {
        String username = authAccountVo.getUsername();
        Integer id = authAccountVo.getId();
        try {
            getUserByName(username);
        } catch (UsernameNotFoundException e) {
            if (!ServiceUtils.update(this, authAccountVo)) {
                return new MessageResponse("修改失败！", 403);
            }
            this.updateUserAuthentication(id);
            return new MessageResponse("修改成功");
        }
        return new MessageResponse("修改失败，用户名已存在", 403);
    }

    public MessageResponse changePassword(Integer userId, String oldPassword, String newPassword) {
        oldPassword = this.encoder.encode(oldPassword);
        newPassword = this.encoder.encode(newPassword);
        if (this.accountMapper.selectOne(new QueryWrapper<AccountVo>().eq("id", userId).eq("password", oldPassword)) == null) {
            return new MessageResponse("原密码错误", 403);
        }
        AccountVo accountVo = new AccountVo();
        accountVo.setId(userId);
        accountVo.setPassword(newPassword);
        this.accountMapper.updateById(accountVo);
        return new MessageResponse("修改成功");
    }

    public MessageResponse delUser(Integer id) {
        if (this.accountMapper.deleteById(id) == 0) {
            return new MessageResponse("未找到用户", 404);
        }
        return new MessageResponse("修改成功");
    }

    public PageableResponse<AccountVo> getUser(Long page, Long num, Authentication authentication) {
        User user = (User) authentication.getPrincipal();
        Integer uid = user.getId();
        Integer id = user.getGroup().getId();
        QueryWrapper<AccountVo> queryWrapper = new QueryWrapper<AccountVo>().gt("role_id", id).or().eq("id", uid);
        Page<AccountVo> users = this.accountMapper.selectPage(new Page<>(page, num), queryWrapper);
        Long l = this.accountMapper.selectCount(queryWrapper);
        return new PageableResponse<>(users.getRecords(), l);
    }

    public void updateUserAuthentication(int id) {
        if (!userContexts.containsKey(id)) {
            return;
        }
        User userById = getUserById(id);
        Authentication newAuth = new UsernamePasswordAuthenticationToken(userById, null, userById.getAuthorities());
        SecurityContext securityContext = userContexts.get(id);
        securityContext.setAuthentication(newAuth);
    }
}
